/**
 * @todo fix jump on swipe-down then pinch close?
 */
import { Point, Vector } from "../../utils/math";
export declare class Lightbox extends HTMLElement {
    opener: HTMLImageElement;
    image: HTMLImageElement;
    wrapper: HTMLElement;
    figure: HTMLElement;
    loader: HTMLImageElement;
    width: number;
    height: number;
    /**
     * Rendered size of the image, after applied transforms.
     */
    size: Point;
    /**
     * Computed transform-origin of the figure, determining how calculations
     * should be done. Set in CSS.
     */
    origin: Point;
    /**
     * The figure's cached `offsetWidth` and `offsetHeight` values.
     */
    offsetSize: Point;
    /**
     * The figure's cached `offsetLeft` and `offsetTop` values.
     */
    offsetPosition: Point;
    /**
     * Current active position of the figure, before applying gesture.
     */
    position: Point;
    /**
     * Averaged position of all pointers on pointer change. Used to determine
     * how much the user has moved the figure since the last pointer up/down.
     */
    downPosition: Point;
    /**
     * Current active scale of the figure, before applying gesture.
     */
    scale: number;
    /**
     * Maximum scale to which the user can zoom before overdrag kicks in. It's
     * not relative to the image's intrinsic size when that size is smaller than
     * 1024 on either width or height and the user can zoom beyond it.
     */
    maxScale: number;
    /**
     * Most optimal zoomed-in scale for the image, accounting for DPR.
     */
    idealScale: number;
    /**
     * Offset from the active position to the user pointer down position. When
     * the user pinches the image, this is used to pull the image such that it
     * scales relative to the center of the finger gesture.
     */
    pull: Vector;
    /**
     * Last effective point of focus, computed from the average of all pointers.
     * When a pinch gesture ends and the figure scale is constrained, this
     * determines the point from which the figure should scale down.
     */
    focusPosition: Point;
    /**
     * Current active angle of the figure, before applying gesture.
     */
    angle: number;
    /**
     * Current active pointers on the screen, used to perform a gesture.
     */
    pointers: {
        id: number;
        point: Point;
        origin: Point;
    }[];
    /**
     * Active average pointer position.
     */
    gesturePosition: Point;
    /**
     * Offset from the `downPosition` to the `gesturePosition`.
     */
    gestureOffset: Vector;
    /**
     * Average distance between pointers on pointer change. Used to later
     * determine if the user is zooming in or out when pinching.
     */
    gestureStartDistance: number;
    /**
     * Active scale difference caused by the user gesture.
     */
    gestureScale: number;
    /**
     * Average angle between pointers on pointer change. Used to later determine
     * how much the user has rotated the figure.
     */
    gestureStartAngle: Vector;
    /**
     * Active rotation by the user gesture.
     */
    gestureAngle: number;
    /**
     * Rectangle that determines where the user can drag the image before
     * overdrag kicks in. Computed with the actual rendered scale of the image.
     */
    bounds: DOMRectReadOnly;
    speed: Vector;
    tickPosition: Point;
    movement: Vector;
    timeScale: number;
    animRatio: number;
    isClosed: boolean;
    isOpening: boolean;
    isSliding: boolean;
    isMoved: boolean;
    isDragged: boolean;
    isRotate: boolean;
    isSnappy: boolean;
    /**
     * Whether the user has scaled the image below 100% during a gesture.
     */
    isScaledDown: boolean;
    isPinchToClose: boolean;
    scaleDirection: number;
    isDoubleTap: boolean;
    doubleTapDistance: number;
    isSwipeDownClose: boolean;
    lastTapUp: PointerEvent;
    constructor();
    getHtml(): string;
    getCss(): string;
    connectedCallback(): void;
    preloadImage(): void;
    getSrc(): string;
    updateDimensions(): void;
    updateOffsets(): void;
    updateImageSize(): void;
    open(): void;
    handleKeyDownCallback: (e: KeyboardEvent) => void;
    handleKeyDown(e: KeyboardEvent): void;
    handleReiszeCallback: () => void;
    handleResize(): void;
    handleClickCallback: (e: PointerEvent) => void;
    handleClick(e: PointerEvent): void;
    handleWheelCallback: (e: WheelEvent) => void;
    handleWheel(e: WheelEvent): void;
    updateSize(): void;
    isExpanded(): boolean;
    isExpandable(): boolean;
    updateImageSrc(): void;
    updateResolutionCallback: () => void;
    /**
     * Resolution is used to increase the image's rendered size (pixels loaded
     * on the GPU) by increasing its width and height. Otherwise, if `transform`
     * is used to scale it up, it gets blurry on mobile devices.
     */
    updateResolution(): void;
    onOpenEndCallback: () => void;
    onOpenEnd(): void;
    handleDownCallback: (e: PointerEvent) => void;
    handleDown(e: PointerEvent): void;
    pointersChange(): void;
    gestureStart(e: PointerEvent): void;
    getPointersAngle(): Vector;
    handleMoveCallback: (e: PointerEvent) => void;
    handleMove(e: PointerEvent): void;
    constrainScale(scale: number): number;
    getScaleOverLimit(amount: number, limit: number): number;
    handleUpCallback: (e: PointerEvent) => void;
    handleUp(e: PointerEvent): void;
    gestureEnd(e: PointerEvent): void;
    clickZoom(): void;
    zoomOut(): void;
    zoomIn(): void;
    handleTickCallback: (delta: number) => void;
    handleTick(delta: number): void;
    stopSliding(): void;
    constrainPoint(p: Point, overdrag?: boolean, retouch?: boolean): void;
    getGestureTransform(): {
        point: Point;
        scale: number;
        angle: number;
    };
    render(): void;
    close(): void;
    /**
     * @todo only needs to be called when image scale or window size changes.
     */
    updateBounds(scale?: number): void;
    getMaxBoundsRect(): DOMRect;
    getOverdrag(amount: number, limit: number): number;
    getBleed(p: Point): number;
    getAveragePoint(): Point;
    getAverageDistance(): number;
}
export declare function createLightbox(img: HTMLImageElement): void;
