import { getClientRect } from "../../utils/dom";
import { clamp } from "../../utils/math";
import { Component } from "../..";
export class Parallax extends Component {
    constructor() {
        super(...arguments);
        this.scrollHandler = this.update.bind(this);
    }
    init() {
        this.reference = this.$options.reference;
        this.reference.addEventListener("scroll", this.scrollHandler);
        this.update();
    }
    update() {
        let value = this.getValue();
        if (this.$options.clamp) {
            value = clamp(value, 0, 1);
        }
        value = this.transform(value);
        this.apply(value);
    }
    getValue() {
        let rect = this.$element.getBoundingClientRect();
        let refRect = getClientRect(this.reference);
        return this.calculate(rect, refRect, this.$options.axis);
    }
    calculate(elRect, refRect, axis) {
        let offset;
        let edge;
        let edgeEnd;
        if (axis === "x") {
            offset = refRect.width;
            edge = elRect.left - refRect.left;
            edgeEnd = elRect.width;
        }
        else {
            offset = refRect.height;
            edge = elRect.top - refRect.top;
            edgeEnd = elRect.height;
        }
        let positionCurrent = offset - edge;
        let positionEnd = edgeEnd + offset;
        return positionCurrent / positionEnd;
    }
    transform(value) {
        return (0.5 - value) * 2;
    }
    apply(value) {
        this.$element.style.setProperty(`--${this.$options.var}`, value.toString());
    }
    destroy() {
        this.reference.removeEventListener("scroll", this.scrollHandler);
    }
}
Parallax.defaults = {
    var: "parallax",
    axis: "y",
    clamp: true,
    reference: window,
};
export default Parallax;
