import { findAnchor } from "../../utils/dom";
import { easeOutQuint } from "../../utils/easings";
import { defaults } from "../../utils/functions";
import { scrollTo } from "../../utils/scroll";
import { Component } from "../..";
export class ScrollTo extends Component {
    constructor() {
        super(...arguments);
        this.clickHandler = this.handleClick.bind(this);
    }
    static clickHandler(options) {
        return (event) => {
            if (event.defaultPrevented) {
                // If the default behavior is prevented, e.g. due to a carousel
                // item being dragged, do nothing. Otherwise, when the drag
                // ends, the page will be scrolled, which is unexpected.
                return false;
            }
            let el = findAnchor(event.target);
            let href = el === null || el === void 0 ? void 0 : el.getAttribute("href");
            if (!href) {
                return false;
            }
            let url = new URL(href, window.location.href);
            if (url.hash &&
                url.origin === window.location.origin &&
                url.pathname === window.location.pathname) {
                let target = document.querySelector(url.hash);
                let config = defaults({
                    target,
                }, options, this.defaults);
                return scrollTo(config);
            }
            else {
                return false;
            }
        };
    }
    init() {
        this.$element.addEventListener("click", this.clickHandler);
    }
    handleClick() {
        scrollTo(this.$options);
    }
    destroy() {
        this.$element.removeEventListener("click", this.clickHandler);
    }
}
ScrollTo.defaults = {
    duration: 1000,
    easing: easeOutQuint,
};
export default ScrollTo;
