import { defaults } from "../../utils/functions";
export class Component {
    constructor(element, options) {
        this._isInit = false;
        this._isDestroyed = false;
        this.$children = [];
        this.$element = element;
        this.$options = defaults(options, this.constructor.defaults);
        this.create();
    }
    static $name(ctor) {
        if (this.components) {
            let names = Object.entries(this.components)
                .filter((entry) => entry[1] === ctor)
                .map((tuple) => tuple[0]);
            if (names.length === 1) {
                return names[0];
            }
            else if (names.length < 1) {
                throw new Error(`${this.name} has no child: ${ctor.name}`);
            }
            else if (names.length > 1) {
                throw new Error(`Subcomponent has multiple names: ${names}`);
            }
        }
        else {
            throw new Error(`Component has no children: ${this.name}`);
        }
    }
    $init() {
        if (!this._isInit) {
            this.$children.forEach((child) => child.$init());
            this.init();
            this._isInit = true;
        }
    }
    _ref(component, remove = false) {
        if (!component.$name)
            return;
        let prop = "$" + component.$name;
        let value = this[prop];
        if (Array.isArray(value)) {
            let index = value.indexOf(component);
            let added = index >= 0;
            if (remove) {
                if (added) {
                    value.splice(index, 1);
                }
            }
            else if (!added) {
                value.push(component);
            }
        }
        else {
            if (remove) {
                if (value === component) {
                    this[prop] = null;
                }
            }
            else {
                this[prop] = component;
            }
        }
    }
    $addChild(component) {
        if (this.$children.indexOf(component) < 0) {
            component.$name = this.constructor.$name(component.constructor);
            component.$parent = this;
            this.$children.push(component);
            this._ref(component);
        }
    }
    $removeChild(component) {
        let index = this.$children.indexOf(component);
        if (index >= 0) {
            component.$parent = null;
            this.$children.splice(index, 1);
            this._ref(component, true);
        }
    }
    $destroy() {
        if (!this._isDestroyed) {
            this.destroy();
            Array.from(this.$children).forEach((child) => {
                child.$destroy();
            });
            if (this.$parent) {
                this.$parent.$removeChild(this);
            }
            this._isDestroyed = true;
        }
    }
    create() { }
    init() { }
    destroy() { }
}
