export function isObject(input) {
    return typeof input === "object" && input !== null;
}
/**
 * Checks for basic user-defined objects.
 * @see https://stackoverflow.com/q/68220676/3130281
 */
export function isPlainObject(input) {
    if (isObject(input)) {
        let p = Object.getPrototypeOf(input);
        return p === null || p === Object.prototype;
    }
    return false;
}
export function defaults(target, ...sources) {
    sources.forEach((obj) => {
        if (isPlainObject(obj)) {
            for (let k in obj) {
                if (obj.hasOwnProperty(k)) {
                    if (isPlainObject(obj[k]) && isPlainObject(target[k])) {
                        defaults(target[k], obj[k]);
                    }
                    else if (!target.hasOwnProperty(k)) {
                        target[k] = obj[k];
                    }
                }
            }
        }
    });
    return target;
}
export function debounce(callback, time) {
    let timer;
    return function (...args) {
        clearTimeout(timer);
        timer = window.setTimeout(() => {
            callback.apply(null, args);
        }, time);
    };
}
export function throttle(callback, time) {
    let stamp = null;
    return function (...args) {
        let now = Date.now();
        if (now - stamp >= time) {
            stamp = now;
            callback.apply(null, args);
        }
    };
}
