import { Emitter } from "../emitter";
import { Point, Vector } from "../math";
export declare type GestureEvent = MouseEvent | TouchEvent;
export declare type Events = {
    start: (event: GestureEvent) => void;
    down: (event: GestureEvent) => void;
    move: (event: GestureEvent) => void;
    up: (event: GestureEvent) => void;
    end: (event: GestureEvent) => void;
};
export declare class Swipe {
    id: number | string;
    origin: Point;
    position: Point;
    positionPrev: Point;
    delta: Vector;
    speed: number;
    constructor(id: number | string, origin: Point);
    offset(): Point;
}
export declare class Gesture extends Emitter<Events> {
    element: Element;
    swipes: Swipe[];
    protected mouseStartHandler: (event: MouseEvent) => void;
    protected mouseMoveHandler: (event: MouseEvent) => void;
    protected mouseEndHandler: (event: MouseEvent) => void;
    protected touchStartHandler: (event: TouchEvent) => void;
    protected touchMoveHandler: (event: TouchEvent) => void;
    protected touchEndHandler: (event: TouchEvent) => void;
    protected tickHandler: (delta: number) => void;
    constructor(element: Element);
    protected getEventPoint(input: MouseEvent | Touch): Point;
    protected touchStart(event: TouchEvent): void;
    protected touchMove(event: TouchEvent): void;
    protected touchEnd(event: TouchEvent): void;
    protected mouseStart(event: MouseEvent): void;
    protected mouseMove(event: MouseEvent): void;
    protected mouseEnd(event: MouseEvent): void;
    protected emitStart(event: GestureEvent): void;
    protected emitMove(event: GestureEvent): void;
    protected emitEnd(event: GestureEvent): void;
    protected handleTick(delta: number): void;
    destroy(): void;
}
