/**
 * Restricts a value to a specified minimum and maximum.
 */
export function clamp(number, min, max) {
    return Math.min(Math.max(number, min), max);
}
export class Point {
    constructor(x = 0, y = 0) {
        this.x = x;
        this.y = y;
        this.set(x, y);
    }
    static average(...points) {
        let pt = new Point();
        points.forEach((p) => pt.addPoint(p));
        pt.x /= points.length;
        pt.y /= points.length;
        return pt;
    }
    clone() {
        return new Point(this.x, this.y);
    }
    set(a, b) {
        this.x = a;
        this.y = b !== null && b !== void 0 ? b : a;
        return this;
    }
    setPoint(p) {
        return this.set(p.x, p.y);
    }
    add(a, b) {
        this.x += a;
        this.y += b !== null && b !== void 0 ? b : a;
        return this;
    }
    addPoint(p) {
        return this.add(p.x, p.y);
    }
    addVector(v) {
        return this.add(v.magnitude * Math.cos(v.direction), v.magnitude * Math.sin(v.direction));
    }
    sub(a, b) {
        this.x -= a;
        this.y -= b !== null && b !== void 0 ? b : a;
        return this;
    }
    subPoint(p) {
        return this.sub(p.x, p.y);
    }
    mul(a, b) {
        this.x *= a;
        this.y *= b !== null && b !== void 0 ? b : a;
        return this;
    }
    mulPoint(p) {
        return this.mul(p.x, p.y);
    }
    dist(p) {
        let x = this.x;
        let y = this.y;
        if (p) {
            x -= p.x;
            y -= p.y;
        }
        return Math.hypot(Math.abs(x), Math.abs(y));
    }
    to(input) {
        return new Vector(this, input);
    }
}
export class Vector {
    constructor(p1, p2) {
        this.magnitude = 0;
        this.direction = 0;
        if (p1) {
            this.set(p1, p2);
        }
    }
    copy() {
        let v = new Vector();
        v.magnitude = this.magnitude;
        v.direction = this.direction;
        return v;
    }
    set(p1, p2) {
        let x = p1.x;
        let y = p1.y;
        if (p2) {
            x = p2.x - x;
            y = p2.y - y;
        }
        this.magnitude = Math.sqrt(Math.pow(y, 2) + Math.pow(x, 2));
        this.direction = Math.atan2(y, x);
    }
    add(vector) {
        let point = new Point();
        point.addVector(this);
        point.addVector(vector);
        this.set(point);
    }
}
